<?php

namespace Realestate\Property\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Realestate\Property\Http\Requests\PropertyRequest;
use Realestate\Property\Interfaces\PropertyRepositoryInterface;
use Realestate\Property\Models\Property;
use Realestate\Property\Forms\Property as Form;

/**
 * APIController  class for property.
 */
class PropertyAPIController extends BaseController
{

    /**
     * Initialize property resource controller.
     *
     * @param type PropertyRepositoryInterface $property
     *
     * @return null
     */
    public function __construct(PropertyRepositoryInterface $property)
    {
        parent::__construct();
        $this->repository = $property;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Realestate\Property\Repositories\Criteria\PropertyResourceCriteria::class);
    }

    /**
     * Display a list of property.
     *
     * @return Response
     */
    public function index(PropertyRequest $request)
    {
        return $this->repository
            ->setPresenter(\Realestate\Property\Repositories\Presenter\PropertyPresenter::class)
            ->paginate();
    }

    /**
     * Display property.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function show(PropertyRequest $request, Property $property)
    {
        return $property->setPresenter(\Realestate\Property\Repositories\Presenter\PropertyListPresenter::class);
        ;
    }

    /**
     * Create new property.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PropertyRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('property::property.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('property/property/' . $property->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('property/property');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the property.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function update(PropertyRequest $request, Property $property)
    {
        try {
            $data = $request->all();

            $property->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('property::property.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('property/property/' . $property->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('property/property/' . $property->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the property.
     *
     * @param Model   $property
     *
     * @return Response
     */
    public function destroy(PropertyRequest $request, Property $property)
    {
        try {
            $property->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('property::property.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('property/property/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('property/property/' . $property->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
